@@ -11,13 +11,13 @@ __PACKAGE__->mk_accessors(
 );
 
 use Shipwright;
-use File::Spec::Functions qw/catfile catdir splitdir/;
+use File::Spec;
 use Shipwright::Util;
 use File::Copy qw/copy move/;
 use File::Temp qw/tempdir/;
 use Config;
 use Hash::Merge;
-use List::MoreUtils qw/firstidx/;
+use List::MoreUtils qw/uniq first_index/;
 
 Hash::Merge::set_behavior('RIGHT_PRECEDENT');
 
@@ -61,7 +61,7 @@ sub run {
 
     }
 
-    confess "we need source arg\n" unless $source;
+    die "we need source arg\n" unless $source;
 
     if ( $self->extra_tests ) {
         $shipwright->backend->import(
@@ -85,7 +85,7 @@ sub run {
                 $self->name($name);
             }
             if ( $self->name !~ /^[-.\w]+$/ ) {
-                confess
+                die
                   qq{name can only have alphanumeric characters, "." and "-"\n};
             }
         }
@@ -132,16 +132,16 @@ sub run {
         my $base = $self->_parent_dir($source);
 
         my $script_dir;
-        if ( -e catdir( $base, '__scripts', $name ) ) {
-            $script_dir = catdir( $base, '__scripts', $name );
+        if ( -e File::Spec->catdir( $base, '__scripts', $name ) ) {
+            $script_dir = File::Spec->catdir( $base, '__scripts', $name );
         }
         else {
-
      # Source part doesn't have script stuff, so we need to create by ourselves.
             $script_dir = tempdir( CLEANUP => 1 );
 
             if ( my $script = $self->build_script ) {
-                copy( $self->build_script, catfile( $script_dir, 'build' ) );
+                copy( $self->build_script,
+                    File::Spec->catfile( $script_dir, 'build' ) );
             }
             else {
                 $self->_generate_build( $source, $script_dir, $shipwright );
@@ -152,11 +152,11 @@ sub run {
         unless ( $self->no_follow ) {
             $self->_import_req( $source, $shipwright, $script_dir );
 
-            if ( -e catfile( $source, '__require.yml' ) ) {
+            if ( -e File::Spec->catfile( $source, '__require.yml' ) ) {
                 move(
-                    catfile( $source,     '__require.yml' ),
-                    catfile( $script_dir, 'require.yml' )
-                ) or confess "move __require.yml failed: $!\n";
+                    File::Spec->catfile( $source,     '__require.yml' ),
+                    File::Spec->catfile( $script_dir, 'require.yml' )
+                ) or die "move __require.yml failed: $!\n";
             }
         }
 
@@ -187,8 +187,7 @@ sub run {
             Hash::Merge::merge( $shipwright->backend->source || {}, $new_url )
         );
 
-        my $new_order = $shipwright->backend->fiddle_order;
-        $shipwright->backend->order( $new_order );
+        $self->_reorder($shipwright);
     }
 
     print "imported with success\n";
@@ -198,18 +197,18 @@ sub run {
 # _import_req: import required dists for a dist
 
 sub _import_req {
-    my $self       = shift;
-    my $source     = shift;
-    my $shipwright = shift;
-    my $script_dir = shift;
+    my $self         = shift;
+    my $source       = shift;
+    my $shipwright   = shift;
+    my $script_dir   = shift;
 
-    my $require_file = catfile( $source, '__require.yml' );
-    $require_file = catfile( $script_dir, 'require.yml' )
-      unless -e catfile( $source, '__require.yml' );
+    my $require_file = File::Spec->catfile( $source, '__require.yml' );
+    $require_file = File::Spec->catfile( $script_dir, 'require.yml' )
+      unless -e File::Spec->catfile( $source, '__require.yml' );
 
     my $dir = $self->_parent_dir($source);
 
-    my $map_file = catfile( $dir, 'map.yml' );
+    my $map_file = File::Spec->catfile( $dir, 'map.yml' );
 
     if ( -e $require_file ) {
         my $req = Shipwright::Util::LoadFile($require_file);
@@ -239,23 +238,22 @@ sub _import_req {
                         next;
                     }
 
-                    $s = catdir( $dir, $s );
+                    $s = File::Spec->catdir( $dir, $s );
 
                     my $script_dir;
-                    if ( -e catdir( $dir, '__scripts', $dist ) ) {
-                        $script_dir = catdir( $dir, '__scripts', $dist );
+                    if ( -e File::Spec->catdir( $dir, '__scripts', $dist ) ) {
+                        $script_dir =
+                          File::Spec->catdir( $dir, '__scripts', $dist );
                     }
                     else {
-                        $script_dir = tempdir(
-                            'shipwright_XXXXXX',
-                            CLEANUP => 1,
-                            TMPDIR  => 1
-                        );
-                        if ( -e catfile( $s, '__require.yml' ) ) {
+                        $script_dir = tempdir( CLEANUP => 1 );
+                        if ( -e File::Spec->catfile( $s, '__require.yml' ) ) {
                             move(
-                                catfile( $s,          '__require.yml' ),
-                                catfile( $script_dir, 'require.yml' )
-                            ) or confess "move $s/__require.yml failed: $!\n";
+                                File::Spec->catfile( $s, '__require.yml' ),
+                                File::Spec->catfile(
+                                    $script_dir, 'require.yml'
+                                )
+                            ) or die "move $s/__require.yml failed: $!\n";
                         }
 
                         $self->_generate_build( $s, $script_dir, $shipwright );
@@ -291,8 +289,10 @@ sub _generate_build {
     my $script_dir = shift;
     my $shipwright = shift;
 
+    chdir $source_dir;
+
     my @commands;
-    if ( -f catfile( $source_dir, 'Build.PL' ) ) {
+    if ( -f 'Build.PL' ) {
         print
 "detected Module::Build build system; generating appropriate build script\n";
         push @commands,
@@ -305,7 +305,7 @@ sub _generate_build {
         # is just a symblic link which can't do things right
         push @commands, "clean: %%PERL%% Build realclean";
     }
-    elsif ( -f catfile( $source_dir, 'Makefile.PL' ) ) {
+    elsif ( -f 'Makefile.PL' ) {
         print
 "detected ExtUtils::MakeMaker build system; generating appropriate build script\n";
         push @commands,
@@ -315,7 +315,7 @@ sub _generate_build {
         push @commands, "install: make install";
         push @commands, "clean: make clean";
     }
-    elsif ( -f catfile( $source_dir, 'configure' ) ) {
+    elsif ( -f 'configure' ) {
         print
 "detected autoconf build system; generating appropriate build script\n";
         @commands = (
@@ -344,7 +344,7 @@ sub _generate_build {
         push @commands, 'clean: ';
     }
 
-    open my $fh, '>', catfile( $script_dir, 'build' ) or confess $@;
+    open my $fh, '>', File::Spec->catfile( $script_dir, 'build' ) or die $@;
     print $fh $_, "\n" for @commands;
     close $fh;
 }
@@ -354,9 +354,53 @@ sub _generate_build {
 sub _parent_dir {
     my $self   = shift;
     my $source = shift;
-    my @dirs   = splitdir($source);
+    my @dirs   = File::Spec->splitdir($source);
     pop @dirs;
-    return catdir(@dirs);
+    return File::Spec->catfile(@dirs);
+}
+
+# _reorder:
+# make some hack for order.
+# move ExtUtils::MakeMaker and Module::Build to the head of cpan dists
+
+sub _reorder {
+    my $self       = shift;
+    my $shipwright = shift;
+    my $order      = $shipwright->backend->order;
+
+    my $first_cpan_index = first_index { /^cpan-/ } @$order;
+
+    unless (
+        (
+            $order->[$first_cpan_index] eq 'cpan-ExtUtils-MakeMaker'
+            && ( ( ( first_index { $_ eq 'cpan-Module-Build' } @$order ) == -1 )
+                || $order->[ $first_cpan_index + 1 ] eq 'cpan-Module-Build' )
+        )
+        || (
+            $order->[$first_cpan_index] eq 'cpan-Module-Build'
+            && (
+                (
+                    ( first_index { $_ eq 'cpan-ExtUtils-MakeMaker' } @$order )
+                    == -1
+                )
+                || $order->[ $first_cpan_index + 1 ] eq
+                'cpan-ExtUtils-MakeMaker'
+            )
+        )
+      )
+    {
+        for my $build (qw/cpan-ExtUtils-MakeMaker cpan-Module-Build/) {
+            my $index = first_index { $build eq $_ } @$order;
+            next if $index == -1;    # $index == -1 if not found
+            if ( $index > $first_cpan_index ) {    # not the 1st cpan dist
+                splice @$order, $first_cpan_index, 0, $build;
+            }
+        }
+    }
+
+    @$order = uniq @$order;
+    $shipwright->backend->order($order);
+
 }
 
 1;
